#include "capture.h"
#include "frame.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>

int sd;
int port = 1234;

void createsocket() {
	struct   sockaddr_in sin;
 
	/* get an internet domain socket */
	if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

	/* complete the socket structure */
	memset(&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port = htons(port);

	/* bind the socket to the port number */
	if (bind(sd, (struct sockaddr *) &sin, sizeof(sin)) == -1) {
		perror("bind");
		exit(1);
	}

	/* show that we are willing to listen */
	if (listen(sd, 5) == -1) {
		perror("listen");
		exit(1);
	}	
}

#define VIDEO_PALETTE_JPEG 21

int main(int argc, char** argv) {
	FRAMEGRABBER* fg = NULL;
	FRAME* frame = NULL;
	int size;
	int cd;
	struct   sockaddr_in pin;
	socklen_t 	 addrlen;
	//int width = 160, height = 120;
	int width = 320, height = 240; int jpeg = 1;
	//int width = 640, height = 480;

	if (argc < 4) {
		printf("I want 4 args: port width height jpeg\n");
		printf("port: TCP port to listen to\n");
		printf("jpeg: set to 1 to not compress the data\n");
		exit(-1);
	}
	port = atoi(argv[1]);
	width = atoi(argv[2]);
	height = atoi(argv[3]);
	jpeg = atoi(argv[4]);

	printf("Port:%d, size=%dx%d jpeg=%d\n", port, width, height, jpeg);
	
	// Open the default device  
	
/*	printf("Grabbing one\n");
	int fd = open("test.jpg", O_WRONLY | O_CREAT);
	fg_grab_frame(fg, frame);
	write(fd, frame->data, size);
	close(fd);*/

	printf("Creating the socket... (%d)\n", port);
	createsocket();

	printf("Opening the device...\n");
	fg = fg_open( NULL );  

	if (jpeg)
		fg_set_format( fg, VIDEO_PALETTE_JPEG );
	else
		fg_set_format( fg, VIDEO_PALETTE_RGB32 );
	fg_set_capture_window( fg, 0, 0, width, height);
	frame = fg_new_compatible_frame( fg );
	size = frame_get_size( frame );

	while (1) {
		printf("Waiting for connection...\n");
		if ((cd = accept(sd, (struct sockaddr *)  &pin, &addrlen)) == -1) {
			perror("accept");
			exit(1);
		}

		short buf[3];
		buf[0] = width;
		buf[1] = height;
		buf[2] = jpeg;
		write(cd, buf, sizeof(buf));
		
		printf("Grabbing...\n");
		while (1) {
			/* Check if a command was received */
			fd_set rfds;
			struct timeval tv;
			int retval = 1;
			while (retval) {
				FD_ZERO(&rfds);
				FD_SET(cd, &rfds);
				tv.tv_sec = 0;
				tv.tv_usec = 1;
				retval = select(cd+1, &rfds, NULL, NULL, &tv);
				if (retval) { /* Data! */
					char cbuf[2];
					read(cd, cbuf, sizeof(cbuf));
					printf("Got a command (%d=%d)!\n", cbuf[0], cbuf[1]);
					switch(cbuf[0]) {
					case 0: fg_set_brightness(fg, cbuf[1]); break;
					case 1: fg_set_hue(fg, cbuf[1]); break;
					case 2: fg_set_contrast(fg, cbuf[1]); break;
					case 3: fg_set_colour(fg, cbuf[1]); break;
					case 4: fg_set_whiteness(fg, cbuf[1]); break;
					}
				}
			}

			fg_grab_frame(fg, frame);
			int msize = size;
			if (jpeg) {
				char* cdata = frame->data;
				/* Find the end */
				for (msize = 0; msize < size-2; msize++) {
					if (cdata[msize] == 0xff && cdata[msize+1] == 0xd9)
						break;
				}
				msize += 2;
			}		
			int len = write(cd, frame->data, msize);
			int data = *((int*)frame->data);
			//printf("TCP wrote %d/%d (%x)\n", len, size, data);
			if (len < 0)
				goto out;
//			printf("%p(%d)\n", frame->data, size);
		}
	out:    ;
	}
	
	frame_release( frame );  
	fg_close( fg );  
	
	return 0;
}
